local Keybindings =  FlexBar2:NewModule("Keybindings");

local BindingCache = {};
GlobalCache = BindingCache;

function Keybindings:ValidateBinding(Key)
    return AceLibrary("AceConsole-2.0").keybindingValidateFunc(Key);
end

Keybindings.ButtonMixin = FlexBar2.OO.Mixin
{
	"SetBinding",	
};

function Keybindings.ButtonMixin:LoadSettings()
    self:SetBinding(self.InfoTable.KeyBinding);
end

function Keybindings.ButtonMixin:SetBinding(Key)
    local SavedBinding = self.InfoTable.KeyBinding;
    FlexBar2:Debug("Setting binding to [" , Key,  "] after verif [", Keybindings:ValidateBinding(Key) ,"] from old binding [", SavedBinding, "]");
    ClearOverrideBindings(self.Frame); 
    if(Keybindings:ValidateBinding(Key)) then
        FlexBar2:Debug("Verif passed");
        SetOverrideBindingClick(self.Frame, nil, Key, self.Frame:GetName());
    end
    if(type(SavedBinding) == "string") then -- Clear the old binding from the cache
        BindingCache[SavedBinding] = nil;
    end
    if(type(Key) == "string") then
        if(type(BindingCache[Key]) == "table") then
            BindingCache[Key]:SetBinding(); -- Clear the binding from the old frame
        end
        BindingCache[Key] = self;
    end
    self.InfoTable.KeyBinding = Key;
end

-- GUI --
local Waterfall = FlexBar2.Waterfall;
local KeyBinding;
function Keybindings.ButtonWaterfall(Action, ButtonName, ButtonTable)
	if(Action == "edit") then
		if(#ButtonTable == 1) then
            KeyBinding = FlexBar2:GetSetting(ButtonTable, "KeyBinding");
            Waterfall:AddControl(
            "type", "label",
            "text", "Keybinding: ");
            Waterfall:AddControl(
            "type", "keybind",
            "width", "50",
            "noNewLine", true,
            "causeRefresh", false,
            "setFunc", function(NewBinding) KeyBinding = NewBinding; end,
            "getFunc", function() return KeyBinding; end);
        end
	elseif(Action == "save") then
        FlexBar2:Debug("KeyBinding: ", KeyBinding);
        local ButtonList = FlexBar2.Buttons;
        for _, ButtonName in ipairs(ButtonTable) do
            ButtonList[ButtonName]:SetBinding(KeyBinding);
        end
    end
end

-- SlashCommands --
local SlashCommands = FlexBar2.SlashCommands;
local function SlashCommand_SetBinding(Params)
    local ButtonTable = FlexBar2:ToButtonTable(Params.button);
    if(#ButtonTable ~= 1) then FlexBar2:Print("Cannot set the same keybinding on multiple buttons"); return; end
    local ButtonList = FlexBar2.Buttons;
    for _, ButtonName in ipairs(ButtonTable) do
       ButtonList[ButtonName]:SetBinding(Params.binding);
    end
end

local function SlashCommand_ClearBinding(Params)
    local ButtonTable = FlexBar2:ToButtonTable(Params.button);
    local ButtonList = FlexBar2.Buttons;
    for _, ButtonName in ipairs(ButtonTable) do
        ButtonList[ButtonName]:SetBinding();
    end
end

SlashCommands:AddCommand("setbinding", SlashCommand_SetBinding, "keybinding", "/FlexBar2 setbinding button=<Button> binding=<Key>");
	SlashCommands:AddParam("setbinding", "button", true, nil, nil, function(Button) return FlexBar2:ButtonsExist(Button); end);
	SlashCommands:AddParam("setbinding", "binding", true, nil, nil, VerifyBinding);

SlashCommands:AddCommand("clearbinding", SlashCommand_ClearBinding, "keybinding", "/FlexBar2 clearbinding button=<Button>");
	SlashCommands:AddParam("clearbinding", "button", true, nil, nil, function(Button) return FlexBar2:ButtonsExist(Button); end);
